#include <stdio.h>

#include "sh.h"
#include "shio.h"
#include "util.h"
#include "parser.h"
#include "run.h"

void	show_rops(struct redir_op	*rop)
{
	if (rop) {
		show_rops(rop->next);
		printf("%d",rop->target_desc);
		switch (rop->dir) {
		case rfappend:
			putchar('>');
			/* fall through */
		case rout:
			putchar('>');
			break;
		case rin:
			putchar('<');
			break;
		}
		switch (rop->src_type) {
		case rnone:
			printf("SHOULD NOT HAPPEN\n");
			break;
		case rfile:
			printf(" %s\n",rop->src.name);
			break;
		case rdesc:
			printf("&%d\n",rop->src.desc);
			break;
		case rpipe:
			printf("|%s\n",rop->src.name);
			break;
		case rclose:
			printf("&-\n");
			break;
		}
	}
}

void	run_command(struct cmd	*cmdp)
{
	int	i;

	/*
	 * in the next iteration, we will:
	 *
	 * first pull off all leading words of form
	 *  varname=value
	 * to export the variable into the environment of the command;
	 * if there is no command, then these variable setting parts
	 * change the variables of this shell.
	 *
	 * for now, we do not handle shell variables.
	 */
	if (!cmdp->nwords) {
		printf("Null command\n");
		return;
	}
	printf("Command: %s has %d arguments in its arglist\n",cmdp->words[0],cmdp->nwords);
	for (i = 0; i < cmdp->nwords; i++) {
		printf("argv[%d] = \"%s\"\n",i,cmdp->words[i]);
	}
	printf("I/O redirection:\n");
	show_rops(cmdp->rop);
	printf("Command runs in the %sground\n\n",cmdp->background ? "back" : "fore");
}
