#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#include "sh.h"
#include "util.h"
#include "shio.h"

/*
 * duplicates a string into a new memory arena allocated from the heap.
 * this routine will succeed or die trying.
 */
char		*dupstr(char	*str)
{
	char	*tmp;
	int	len;

	if (!str) return str; /* should not be necessary */
	tmp = malloc((len = strlen(str))+1);
	if (!tmp) {
		perror(SHNAME ": dupstr");
		desc_printf(2,SHNAME ": out of memory duplicating a %d byte string\n",len);
		exit(1);
	}
	strcpy(tmp,str);
	return tmp;
}

/*
 * useful, but doesn't give good error reporting as to where the problem
 * occurred.
 */
void		*safe_malloc(size_t	nbytes)
{
	void	*p = malloc(nbytes);

	if (!p) {
		perror(SHNAME ": safe_malloc");
		exit(1);
	}
	return p;
}

void		strbuf_constr(struct strbuf	*sbp)
{
	sbp->used = 0;
	sbp->size = 0;
	sbp->buf = 0;
}

struct strbuf	*strbuf_constr_ptr(void)
{
	struct strbuf	*sbp = safe_malloc(sizeof *sbp);
	strbuf_constr(sbp);
	return sbp;
}

void		strbuf_destr(struct strbuf	*sbp)
{
	if (sbp->buf) free(sbp->buf);
	sbp->size = sbp->used = 0;
	sbp->buf = 0;
}

void		strbuf_destr_ptr(struct strbuf	*sbp)
{
	strbuf_destr(sbp);
	free(sbp);
}

char		*strbuf_take_buf(struct strbuf	*sbp)
{
	char	*buf;

	buf = sbp->buf;
	sbp->buf = 0;
	sbp->size = sbp->used = 0;
	return buf;
}

void		strbuf_add_ch(struct strbuf	*sbp,
			      char		ch)
{
	char	*nbuf;
	int	nsize;

	if (sbp->used == sbp->size) {
		if (sbp->size == 0)
			nsize = 128;
		else
			nsize = 2 * sbp->size;
		nbuf = realloc(sbp->buf,nsize);
		if (!nbuf) {
			perror("strbuf_add");
			desc_printf(2,"out of memory expanding string buffer\n");
			return;
		}
		/* nbuf != NULL */
		sbp->buf = nbuf;
		sbp->size = nsize;
	}
	/* used < size */
	sbp->buf[sbp->used++] = ch;
}

int		strbuf_nchars(struct strbuf	*sbp)
{
	return sbp->used;
}

void		strbuf_clear(struct strbuf	*sbp)
{
	sbp->used = 0;
}
